@@ -12,6 +12,7 @@ class AutoConvHandler:
 		self.prev_state = None
 		self.curr_state = conversation.start
 		self.update, self.context = None, None
+		self._bkup_state_routes,self._bkup_state_keyboard = None,None
 
 	def _build_keyboard(self,state):
 		'''Build Keyboard for callback state'''
@@ -26,12 +27,15 @@ class AutoConvHandler:
 
 	def _change_state(self,telegram_id,data):
 		'''Set variables for next state'''
+		data_context = self.context.user_data.get(telegram_id)
 		state = self.conversation.get_state(self.context.user_data.get(telegram_id).get('state'))
 		value = state.callback[0][data] if state.callback and state.callback[0].get(data) else data
 		if state != self.conversation.end: self.context.user_data.get(telegram_id).get('data').update({state.name:value})
 		new_state = self._next_state(state,data)
-		self.prev_state = self.curr_state; self.curr_state = new_state
-		self.context.user_data.get(telegram_id).update({'state':new_state.name,'error':False})
+		self.prev_state,self.curr_state = self.curr_state,new_state
+		data_context.update({'prev_state':self.prev_state.name,'state':new_state.name})
+		if new_state.text: data_context.update({'error':False})
+		elif data_context.get('error') != None: data_context.pop('error')
 		return new_state
 
 	def _wrong_text(self):
@@ -58,7 +62,28 @@ class AutoConvHandler:
 		ro,de,ba = state.routes(self.update,self.context)
 		self.conversation.add_routes(state,ro,de,ba)
 
-	def _build_dynamic_stuff(self,state):
+	def _build_dynamic_stuff(self,state): 
+		data = self.context.user_data.get(self.update.effective_chat.id)
+		if state.list:
+			if (state_l := data.get('list')):
+				i = int(data.get('list_i'))
+				new_i = state_l.index(data.get('data').get(state.name)) if state.list_all else (i-1,i+1)[data.get('data').get(state.name) == state.list_buttons[1]]
+				data.update({'list_i':new_i})
+			else:
+				state_l = state.list(self.update,self.context)
+				self._bkup_state_routes,self._bkup_state_keyboard = self.conversation.routes.get(state.name),state.callback
+				basic_routes = {k+len(state_l):v for k,v in self.conversation.routes.get(state.name).items()}
+				basic_keyboard = {k+len(state_l):v for k,v in state.callback[0].items()}
+				arrows_buttons = {i:b for i,b in enumerate(state.list_buttons)} if not state.list_all else {i:b for i,b in enumerate(state_l)}
+				state.add_keyboard({**arrows_buttons,**basic_keyboard},size=(2,len(basic_routes))) if not state.list_all else state.add_keyboard({**arrows_buttons,**basic_keyboard},size=(len(state_l),len(basic_routes)))
+				self.conversation.add_routes(state,basic_routes,default=state)
+				new_i = state.list_start
+				data.update({'list':state_l,'list_i':new_i})
+			if not state.list_all:
+				if new_i in (0,len(state_l)-1): state.callback[0].pop((1,0)[not new_i])
+				if new_i in (1,len(state_l)-2): state.callback[0].update({i:b for i,b in enumerate(state.list_buttons)})
+				if new_i in (0,1,len(state_l)-2,len(state_l)-1): state.callback = dict(sorted(state.callback[0].items())),((2,1)[new_i not in (1,len(state_l)-2)],*state.callback[1][1:])
+		elif data.get('list'): data.pop('list'),data.pop('list_i')
 		ret = state.action(self.update,self.context) if state.action else None
 		if state.routes: self._build_dynamic_routes(state)
 		if state.build: 
@@ -84,11 +109,15 @@ class AutoConvHandler:
 		self.update = update
 		self.context = context
 		telegram_id = self.update.effective_chat.id
+		# restore dynamic list state
+		if self.prev_state and self.prev_state.list and self.curr_state != self.prev_state:
+			self.prev_state.add_keyboard(*self._bkup_state_keyboard)
+			self.conversation.add_routes(self.prev_state,self._bkup_state_routes)
 		# start
 		if not self.context.user_data.get(telegram_id):
 			if delete_first: update.message.delete()
 			state = self.conversation.start
-			self.context.user_data.update({telegram_id:{'state':state.name,'error':False,'data':{}}})
+			self.context.user_data.update({telegram_id:{'prev_state':None,'state':state.name,'error':False,'data':{}}})
 			keyboard,reply_msg = self._build_dynamic_stuff(state)
 			msg = self.update.message.reply_text(f'{reply_msg}',reply_markup=keyboard,parse_mode=state.mode)
 			self.context.user_data.get(telegram_id).update({'bot-msg':msg})
