@@ -1,3 +1,4 @@
+from functools import reduce
 from typing import Union,Callable,Optional,_GenericAlias,_VariadicGenericAlias
 
 def type_check(func):
@@ -24,6 +25,9 @@ class State:
 		self.webpage_preview = webpage_preview
 		self.routes = None
 		self.custom = None
+		self.list = None
+		self.list_all = None
+		self.list_start = None
 	
 	def __str__(self):
 		return f'State <{self.name}>'
@@ -33,6 +37,7 @@ class State:
 		'''Add inline keyboard for the state'''
 		if isinstance(keyboard,list): keyboard = dict(enumerate(keyboard))
 		if size and sum(size) != len(keyboard): raise ValueError(f'Keyboard length ({len(keyboard)}) must be the same size as the sum of row size ({sum(size)}).')
+		if size: size = reduce(lambda x,y:x+y,map(lambda x:[x] if x < 9 else [8]*(x//8)+([x%8],[])[not x%8],size))
 		elem_n = len(keyboard)
 		if not size: size = [max_row for _ in range(elem_n//max_row)] + ([r,] if (r := elem_n % max_row) else [])
 		self.callback = (keyboard,tuple(size))
@@ -60,6 +65,14 @@ class State:
 		self.custom = function
 
 	@type_check
-	def add_dynamic_routes(self,function):
+	def add_dynamic_routes(self,function:Callable):
 		'''Add function to create dynamic routes | must return 3 value (routes,default,back)'''
 		self.routes = function
+
+	@type_check
+	def add_dynamic_list(self,function:Callable,start:Optional[int]=0,left_button:Optional[str]='<',right_button:Optional[str]='>',all_pages:Optional[bool]=False):
+		'''Add function to create a list with page'''
+		self.list = function
+		self.list_buttons = [left_button,right_button]
+		self.list_all = all_pages
+		self.list_start = start
